"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.inspectAndRecommendStepFunctionsInstrumentation = exports.updateDefinitionString = exports.isStepFunctionInvocation = exports.isDefaultLambdaApiStep = exports.isSafeToModifyStepFunctionInvoctation = exports.isSafeToModifyStepFunctionLambdaInvocation = void 0;
function isSafeToModifyStepFunctionLambdaInvocation(parameters) {
    if (typeof parameters !== "object") {
        return false;
    }
    if (!parameters.hasOwnProperty("Payload.$")) {
        return true;
    }
    else {
        if (parameters["Payload.$"] === "$") {
            // $ % is the default original unchanged payload
            return true;
        }
    }
    return false;
}
exports.isSafeToModifyStepFunctionLambdaInvocation = isSafeToModifyStepFunctionLambdaInvocation;
// Truth table
// Input                    | Expected
// -------------------------|---------
// Empty object             | true
// undefined                | true
// not object               | false
// object without CONTEXT.$ | true
// object with CONTEXT.$    | false
function isSafeToModifyStepFunctionInvoctation(parameters) {
    if (typeof parameters !== "object") {
        return false;
    }
    if (!parameters.hasOwnProperty("Input")) {
        return true;
    }
    if (typeof parameters.Input !== "object") {
        return false;
    }
    if (!parameters.Input.hasOwnProperty("CONTEXT.$")) {
        return true;
    }
    return false;
}
exports.isSafeToModifyStepFunctionInvoctation = isSafeToModifyStepFunctionInvoctation;
function isDefaultLambdaApiStep(resource) {
    // default means not legacy lambda api
    if (resource === undefined) {
        return false;
    }
    if (resource === "arn:aws:states:::lambda:invoke") {
        // Legacy Lambda API resource.startsWith("arn:aws:lambda"), but it cannot inject context obj.
        return true;
    }
    return false;
}
exports.isDefaultLambdaApiStep = isDefaultLambdaApiStep;
function isStepFunctionInvocation(resource) {
    if (resource === undefined) {
        return false;
    }
    return resource.startsWith("arn:aws:states:::states:startExecution");
}
exports.isStepFunctionInvocation = isStepFunctionInvocation;
function parseDefinitionObject(definitionString) {
    if (!(typeof definitionString === "object" && "Fn::Sub" in definitionString && definitionString["Fn::Sub"].length > 0)) {
        throw new Error("unexpected definitionString");
    }
    const unparsedDefinition = definitionString["Fn::Sub"] ? definitionString["Fn::Sub"][0] : ""; // index 0 should always be a string of step functions definition
    if (unparsedDefinition === "") {
        throw new Error("no definition string found in DefinitionString");
    }
    const definitionObj = JSON.parse(unparsedDefinition);
    return definitionObj;
}
// Updates the definitionString of a step function to include trace context as appropriate for a Lambda invocation or a nested step function invocation.
// definitionString can either be an object or a naked string, so we need to return the same and explicitly modify the Resource in span-link.ts
function updateDefinitionString(definitionString, serverless, stateMachineName) {
    let definitionObj;
    if (typeof definitionString !== "string") {
        // definitionString is a {"Fn::Sub": (string | object)[]}
        try {
            definitionObj = parseDefinitionObject(definitionString);
        }
        catch (error) {
            serverless.cli.log("Unable to update StepFunction definition. " + error);
            return definitionString;
        }
    }
    else {
        definitionObj = JSON.parse(definitionString);
    }
    const states = definitionObj.States;
    for (const stepName in states) {
        if (states.hasOwnProperty(stepName)) {
            const step = states[stepName];
            if (!isDefaultLambdaApiStep(step === null || step === void 0 ? void 0 : step.Resource) && !isStepFunctionInvocation(step === null || step === void 0 ? void 0 : step.Resource)) {
                // only inject context into default Lambda API steps and Step Function invocation steps
                continue;
            }
            if (isDefaultLambdaApiStep(step === null || step === void 0 ? void 0 : step.Resource)) {
                if (typeof step.Parameters === "object") {
                    if (isSafeToModifyStepFunctionLambdaInvocation(step.Parameters)) {
                        step.Parameters["Payload.$"] = "States.JsonMerge($$, $, false)";
                        serverless.cli.log(`JsonMerge Step Functions context object with payload in step: ${stepName} of state machine: ${stateMachineName}.`);
                    }
                    else {
                        serverless.cli.log(`[Warn] Parameters.Payload has been set. Merging traces failed for step: ${stepName} of state machine: ${stateMachineName}`);
                    }
                }
            }
            else if (isStepFunctionInvocation(step === null || step === void 0 ? void 0 : step.Resource)) {
                if (isSafeToModifyStepFunctionInvoctation(step === null || step === void 0 ? void 0 : step.Parameters)) {
                    if (step.Parameters && !step.Parameters.Input) {
                        step.Parameters.Input = {};
                    }
                    step.Parameters.Input["CONTEXT.$"] = "States.JsonMerge($$, $, false)";
                }
            }
        }
    }
    if (typeof definitionString !== "string") {
        definitionString["Fn::Sub"][0] = JSON.stringify(definitionObj); // writing back to the original JSON created by Serverless framework
    }
    else {
        definitionString = JSON.stringify(definitionObj);
    }
    return definitionString; // return the definitionString so it can be written to the Resource in span-link.ts
}
exports.updateDefinitionString = updateDefinitionString;
function inspectAndRecommendStepFunctionsInstrumentation(serverless) {
    var _a;
    const stepFunctions = Object.values(((_a = serverless.service.stepFunctions) === null || _a === void 0 ? void 0 : _a.stateMachines) || {});
    if (stepFunctions.length !== 0) {
        serverless.cli.log(`Uninstrumented Step Functions detected in your serverless.yml file. If you would like to see Step Functions traces, please see details of 'enableStepFunctionsTracing' and 'mergeStepFunctionAndLambdaTraces' variables in the README (https://github.com/DataDog/serverless-plugin-datadog/)`);
    }
}
exports.inspectAndRecommendStepFunctionsInstrumentation = inspectAndRecommendStepFunctionsInstrumentation;
//# sourceMappingURL=step-functions-helper.js.map